import { Plugin, Transaction } from 'prosemirror-state';
import { InputRule } from 'prosemirror-inputrules';
import { Editor } from './Editor';
import { GlobalAttributes } from './types';
export interface ExtensionConfig<Options = any, Commands = {}> {
    /**
     * Name
     */
    name: string;
    /**
     * Default options
     */
    defaultOptions?: Options;
    /**
     * Global attributes
     */
    addGlobalAttributes?: (this: {
        options: Options;
    }) => GlobalAttributes | {};
    /**
     * Commands
     */
    addCommands?: (this: {
        options: Options;
        editor: Editor;
    }) => Commands;
    /**
     * Keyboard shortcuts
     */
    addKeyboardShortcuts?: (this: {
        options: Options;
        editor: Editor;
    }) => {
        [key: string]: any;
    };
    /**
     * Input rules
     */
    addInputRules?: (this: {
        options: Options;
        editor: Editor;
    }) => InputRule[];
    /**
     * Paste rules
     */
    addPasteRules?: (this: {
        options: Options;
        editor: Editor;
    }) => Plugin[];
    /**
     * ProseMirror plugins
     */
    addProseMirrorPlugins?: (this: {
        options: Options;
        editor: Editor;
    }) => Plugin[];
    /**
     * The editor is ready.
     */
    onCreate?: ((this: {
        options: Options;
        editor: Editor;
    }) => void) | null;
    /**
     * The content has changed.
     */
    onUpdate?: ((this: {
        options: Options;
        editor: Editor;
    }) => void) | null;
    /**
     * The selection has changed.
     */
    onSelection?: ((this: {
        options: Options;
        editor: Editor;
    }) => void) | null;
    /**
     * The editor state has changed.
     */
    onTransaction?: ((this: {
        options: Options;
        editor: Editor;
    }, props: {
        transaction: Transaction;
    }) => void) | null;
    /**
     * The editor is focused.
     */
    onFocus?: ((this: {
        options: Options;
        editor: Editor;
    }, props: {
        event: FocusEvent;
    }) => void) | null;
    /**
     * The editor isn’t focused anymore.
     */
    onBlur?: ((this: {
        options: Options;
        editor: Editor;
    }, props: {
        event: FocusEvent;
    }) => void) | null;
    /**
     * The editor is destroyed.
     */
    onDestroy?: ((this: {
        options: Options;
        editor: Editor;
    }) => void) | null;
}
export declare class Extension<Options = any, Commands = any> {
    #private;
    type: string;
    config: Required<ExtensionConfig>;
    options: Options;
    constructor(config: ExtensionConfig<Options, Commands>);
    static create<O, C>(config: ExtensionConfig<O, C>): Extension<O, C>;
    configure(options?: Partial<Options>): Extension<Options, Commands>;
    extend<ExtendedOptions = Options, ExtendedCommands = Commands>(extendedConfig: Partial<ExtensionConfig<ExtendedOptions, ExtendedCommands>>): Extension<ExtendedOptions, ExtendedCommands>;
}
