import { Node as ProseMirrorNode, ParseOptions } from 'prosemirror-model';
import { EditorView, Decoration, NodeView, EditorProps } from 'prosemirror-view';
import { EditorState, Transaction } from 'prosemirror-state';
import { Extension } from './Extension';
import { Node } from './Node';
import { Mark } from './Mark';
import { Editor } from './Editor';
import { AllExtensions } from '.';
export declare type Extensions = (Extension | Node | Mark)[];
export interface EditorOptions {
    element: Element;
    content: EditorContent;
    extensions: Extensions;
    injectCSS: boolean;
    autofocus: FocusPosition;
    editable: boolean;
    editorProps: EditorProps;
    parseOptions: ParseOptions;
    enableInputRules: boolean;
    enablePasteRules: boolean;
    onCreate: () => void;
    onUpdate: () => void;
    onSelection: () => void;
    onTransaction: (props: {
        transaction: Transaction;
    }) => void;
    onFocus: (props: {
        event: FocusEvent;
    }) => void;
    onBlur: (props: {
        event: FocusEvent;
    }) => void;
    onDestroy: () => void;
}
export declare type EditorContent = string | JSON | null;
export declare type Command = (props: {
    editor: Editor;
    tr: Transaction;
    commands: SingleCommands;
    can: () => CanCommands;
    chain: () => ChainedCommands;
    state: EditorState;
    view: EditorView;
    dispatch: ((args?: any) => any) | undefined;
}) => boolean;
export declare type CommandSpec = (...args: any[]) => Command;
export declare type Attribute = {
    default: any;
    rendered?: boolean;
    renderHTML?: ((attributes: {
        [key: string]: any;
    }) => {
        [key: string]: any;
    } | null) | null;
    parseHTML?: ((element: HTMLElement) => {
        [key: string]: any;
    } | null) | null;
};
export declare type Attributes = {
    [key: string]: Attribute;
};
export declare type ExtensionAttribute = {
    type: string;
    name: string;
    attribute: Required<Attribute>;
};
export declare type GlobalAttributes = {
    types: string[];
    attributes: {
        [key: string]: Attribute;
    };
}[];
export declare type PickValue<T, K extends keyof T> = T[K];
export declare type UnionToIntersection<U> = (U extends any ? (k: U) => void : never) extends ((k: infer I) => void) ? I : never;
export declare type Diff<T extends keyof any, U extends keyof any> = ({
    [P in T]: P;
} & {
    [P in U]: never;
} & {
    [x: string]: never;
})[T];
export declare type Overwrite<T, U> = Pick<T, Diff<keyof T, keyof U>> & U;
export declare type AnyObject = {
    [key: string]: any;
};
export declare type NodeViewRendererProps = {
    editor: Editor;
    node: ProseMirrorNode;
    getPos: (() => number) | boolean;
    HTMLAttributes: {
        [key: string]: any;
    };
    decorations: Decoration[];
    extension: Node;
};
export declare type NodeViewRenderer = (props: NodeViewRendererProps) => (NodeView | {});
export declare type UnfilteredCommands = {
    [Item in keyof AllExtensions]: AllExtensions[Item] extends Extension<any, infer ExtensionCommands> ? ExtensionCommands : AllExtensions[Item] extends Node<any, infer NodeCommands> ? NodeCommands : AllExtensions[Item] extends Mark<any, infer MarkCommands> ? MarkCommands : never;
};
export declare type ValuesOf<T> = T[keyof T];
export declare type KeysWithTypeOf<T, Type> = ({
    [P in keyof T]: T[P] extends Type ? P : never;
})[keyof T];
export declare type AllCommands = UnionToIntersection<ValuesOf<Pick<UnfilteredCommands, KeysWithTypeOf<UnfilteredCommands, {}>>>>;
export declare type SingleCommands = {
    [Item in keyof AllCommands]: AllCommands[Item] extends (...args: any[]) => any ? (...args: Parameters<AllCommands[Item]>) => boolean : never;
};
export declare type ChainedCommands = {
    [Item in keyof AllCommands]: AllCommands[Item] extends (...args: any[]) => any ? (...args: Parameters<AllCommands[Item]>) => ChainedCommands : never;
} & {
    run: () => boolean;
};
export declare type CanCommands = SingleCommands & {
    chain: () => ChainedCommands;
};
export declare type FocusPosition = 'start' | 'end' | number | boolean | null;
export declare type Range = {
    from: number;
    to: number;
};
export declare type Predicate = (node: ProseMirrorNode) => boolean;
